##############################################################################
#
# Copyright (c) 2010 ViFiB SARL and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


import hashlib
import os
import posixpath
import re
import urllib
import urlparse
from download import check_md5sum
from slapos.libnetworkcache import NetworkcacheClient, UploadError


_md5_re = re.compile(r'md5=([a-f0-9]+)')


def _get_md5_from_url(url):
  match = _md5_re.search(url)
  if match:
    return match.group(1)
  return None


def _update_network_cached_log(path, url):
    """Update the networkcached.log file.

    Adding the last key and URL which has been retrieved from shacache server.

    """
    f = open(path, 'r')
    try:
      key = hashlib.sha512(f.read()).hexdigest()
    finally:
      f.close()

    # Keep the log
    map_path = os.path.join(os.environ.get('PWD'), '.networkcached.log')
    mode = 'r+'
    if not os.path.exists(map_path):
        mode = 'w+'
    f = open(map_path, mode)
    try:
        flag = False
        for line in f:
            if url in line:
                flag = True
                break
        if not flag:
          f.seek(0, 2)
          f.write('%s %s\n' % (key, url))
          f.truncate(f.tell())
    finally:
        f.close()


def select_sha512sum_from_shadir(sha_dir, url, logger):
    """It must return the sha512sum if exists on selected directory. """
    # On buildout.cfg the option sha-dir is the complete link including the
    # directory key, so we must to extract it to use the libnetworkcache
    # correctly.
    directory_key = sha_dir.split('/')[-1]
    sha_dir_without_directory_key = '/'.join(sha_dir.split('/')[:-1])

    try:
      nc = NetworkcacheClient(shadir=sha_dir_without_directory_key)
      data_list = nc.select(directory_key)
    except (IOError, UploadError), e:
      logger.info('Could not retrieve the information from SHADIR. %s' % e)
      return None

    urlmd5 = hashlib.md5(url).hexdigest()
    for data in data_list:
      information_dict, signature = data
      if information_dict.get('urlmd5') == urlmd5:
        return information_dict.get('sha512')

    return None


def download_network_cached(sha_dir, sha_cache, path, url, logger, md5sum=None):
    """Download from a network cache provider

    If something fail (providor be offline, or hash_string fail), we ignore
    network cached files.

    return True if download succeeded.
    """
    if sha_cache in (None, '',):
        # Not able to use network cache
        return False
    if md5sum is None:
        md5sum = _get_md5_from_url(url)


    sha512sum = select_sha512sum_from_shadir(sha_dir, url, logger)
    if sha512sum is None:
      return False

    sha_cache_url = os.path.join(sha_cache, sha512sum)
    logger.info('Downloading from network cache %s' % sha_cache_url)
    try:
        nc = NetworkcacheClient(shacache=sha_cache)
        nc.download(path, sha512sum)
        if not check_md5sum(path, md5sum):
            logger.info('MD5 checksum mismatch downloading %r' % sha_cache_url)
            return False
    except IOError, e:
        logger.info('Fail to download from network cache %s: %s' % \
                                                  (sha_cache_url, str(e)))
        return False
    return True


def upload_network_cached(sha_cache, sha_dir, external_url, path, logger):
    """Upload file to a shacache server"""
    if sha_cache in [None, ''] or sha_dir in [None, '']:
        logger.debug(
             'Upload cache ignored, shacache or shadir was not provided!')
        return False

    # On buildout.cfg the option sha-dir is the complete link including the
    # directory key, so we must to extract it to use the libnetworkcache
    # correctly.
    directory_key = sha_dir.split('/')[-1]
    sha_dir_without_directory_key = '/'.join(sha_dir.split('/')[:-1])

    f = open(path, 'r')
    try:

        kw = dict(file_name=os.path.basename(external_url),
                  urlmd5=hashlib.md5(external_url).hexdigest())

        nc = NetworkcacheClient(shacache=sha_cache,
                                shadir=sha_dir_without_directory_key)
        data = nc.upload(f, directory_key, **kw)
        _update_network_cached_log(path, external_url)
    except (IOError, UploadError), e:
        logger.info('Fail to upload file. %s' % \
                                                  (str(e)))
        return False

    finally:
      f.close()

    return True


def get_filename_from_url(url):
    """Inspired how pip get filename from url.
    """
    url = url.split('#', 1)[0]
    url = url.split('?', 1)[0]
    url = url.rstrip('/')
    name = posixpath.basename(url)
    assert name, (
           'URL %r produced no filename' % url)
    return name
